<?php
namespace Teknisa\OPL\Modules\Consolidacao\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\Libs\Util\DateUtil;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\OPL\Util\Attachment;
use Teknisa\OPL\Util\OplUtilities;
use Teknisa\OPL\Modules\Consolidacao\Services\Sup07200LancamentoDePedidosService;

class Sup07200LancamentoDePedidosReport extends \Zeedhi\Framework\Controller\Report {
    protected $entityManager;
    protected $birtStrategy;
    protected $newCode;
    protected $environment;
    protected $connection;
    protected $dataSourceManager;
    protected $qr2Strategy;
    protected $logEmail;
    protected $cookieController;
    protected $oplUtilities;
    protected $globalReports = [];

    public function __construct($reportService, $entityManager, $reportStrategies, $newCode, $environment, $connection, $dataSourceManager, $qr2Strategy, $cookieController, OplUtilities $oplUtilities, Sup07200LancamentoDePedidosService $service) {
        $this->entityManager     = $entityManager;
        $this->reportStrategies  = $reportStrategies;
        $this->reportService     = $reportService;
        $this->newCode           = $newCode;
        $this->environment       = $environment;
        $this->connection        = $connection;
        $this->dataSourceManager = $dataSourceManager;
        $this->qr2Strategy       = $qr2Strategy;
        $this->cookieUtil        = $cookieController;
        $this->oplUtilities      = $oplUtilities;
		$this->service           = $service;
        $this->logEmail          = array('REPORT' => array(), 'SUCCESS' => array(), 'ERROR' => array());
    }
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        ini_set('default_socket_timeout', 10000);
        
        $row = $request->getRow();
        $this->stNomeRel = $this->getReportName($row);
        $this->reportMapping[$this->stNomeRel] = array(
            "strategy" => isset($row["STRATEGY"]) ? $row["STRATEGY"] : "QR2",
            "parameterMapping" => array(
                "P_CDCLIENTE"       => "P_CDCLIENTE",
                "P_TCDCLIENTE"      => "P_TCDCLIENTE",
                "P_CDFILIOL"        => "P_CDFILIOL",
                "P_TCDFILIOL"       => "P_TCDFILIOL",
                "L_ROTA"            => "L_ROTA",
                "L_ROTA1"           => "L_ROTA1",
                "P_ROTA"            => "P_ROTA",
                "P_CDFILIAL"        => "P_CDFILIAL",
                "P_TCDFILIAL"       => "P_TCDFILIAL",
                "P_DTENTRINI"       => "P_DTENTRINI",
                "P_DTENTRFIN"       => "P_DTENTRFIN",
                "P_TDTENTR"         => "P_TDTENTR",
                "P_DTEMBINI"        => "P_DTEMBINI",
                "P_DTEMBFIN"        => "P_DTEMBFIN",
                "P_TDTEMB"          => "P_TDTEMB",
                "P_SUBGRUPO"        => "P_SUBGRUPO",
                "P_CDGRUPPROD"      => "P_CDGRUPPROD",
                "P_CDPRODINI"       => "P_CDPRODINI",
                "P_CDPRODFIN"       => "P_CDPRODFIN",
                "P_CDPRODFINTEXT"   => "P_CDPRODFINTEXT",
                "L_PRODUTO"         => "L_PRODUTO",
                "P_PRODUTO"         => "P_PRODUTO",
                "L_OPERADOR"        => "L_OPERADOR",
                "L_OPERADOR1"       => "L_OPERADOR1",
                "P_OPERADOR"        => "P_OPERADOR",
                "P_NMPRODINI"       => "P_NMPRODINI",
                "P_NMPRODFIN"       => "P_NMPRODFIN",
                "P_IDSITUACAO"      => "P_IDSITUACAO",
                "P_IDBAIXEXOL"      => "P_IDBAIXEXOL",
                "P_IDBAIXEXSAI"     => "P_IDBAIXEXSAI",
                "P_PEDEXOLEXTR"     => "P_PEDEXOLEXTR",
                "P_CDLOTENVITOL"    => "P_CDLOTENVITOL",
                "P_NRSEQRUPTURAOL"  => "P_NRSEQRUPTURAOL",
                "P_TNRSEQRUPTURAOL" => "P_TNRSEQRUPTURAOL"
            )
        );
    }
    
    public function afterProcessReport(DTO\Request\Row $request, DTO\Response $response, $report) {
        $row = $request->getRow();
        
        $this->globalReports[$row["reportLabel"]] = $report;
    }
    
    public function enviaEmailEmissao(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $dadosEmail = $row["DADOS_EMAIL"];
        $currentRow = $row["CURRENT_ROW"];
        
        $isEmailSend = false;
        $localResponse = new DTO\Response();
        
        try {
            $messages = [];
            
            $enderecos = explode(";", $dadosEmail["EMAILS"]);
            
            if(empty($enderecos[0])) {
                $this->logEmail['ERROR'][] = array("MESSAGE" => "EMAIL_N_REGISTERED");
            } else {
                $this->qr2Strategy->setQr2Type($dadosEmail["EXTENSAO"]);
                
                $this->createRemoteReport(new DTO\Request\Row(new DTO\Row($currentRow), "", "", ""), $localResponse);
                
                $isEmailSend = self::sendEmail($currentRow, $dadosEmail, $enderecos, $currentRow["reportLabel"]);
            }
            
            if($isEmailSend) {
                $this->logEmail['SUCCESS'][] = array("MESSAGE" => "EMAIL_SEND_SUCCESS");
            } else {
                $this->logEmail['ERROR'][] = array("MESSAGE" => "EMAIL_SEND_FAILED");
            }
            
            $response->addMessage(new Message($this->logEmail, 'W'));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    private function sendEmail($row, $dadosEmail, $enderecos, $reportName = null) {
        $this->logEmail["REPORT"][] = $this->globalReports;
        $this->cookieUtil->setCookie($row, $dadosEmail);
        
        $result = self::tratarMensagemCorpoEmail($row, $dadosEmail, $reportName);
        
        $message          = $result[0];
        $attachments      = $result[1];
        $clearAttachments = $result[2];
        
        $messageSendMail = $this->oplUtilities->messageSendMail(null, $dadosEmail["ASSUNTO"], $message, 'TECFOOD');
        $enderecosCopia = false;
        
        if($dadosEmail['SENDOPERATOR'] === 'S') {
            $enderecosCopia[] = $this->service->getCurrentEmail();
        }
        
        if(empty($enderecos[0])) {
            $this->logEmail['ERROR'][] = array("MESSAGE" => "EMAIL_N_REGISTERED");
        } else {
            try{
                return $this->oplUtilities->enviaEmail('', $dadosEmail["ASSUNTO"], $enderecos, $messageSendMail, $enderecos, $enderecosCopia, $attachments, $clearAttachments);
            } catch (\Exception $e) {
                return false;
            }
        }
    }
    
    private function tratarMensagemCorpoEmail($row, $dadosEmail, $reportName) {
        $message = $dadosEmail['MENSAGEM'];
        $attachments = [];
        $attOnBody = '';
        $clearAttachments = false;
        
        foreach ($this->globalReports as $name => $report) {
            $fileName = $dadosEmail["NOME_ARQUIVO"] . '.' . strtolower($dadosEmail["EXTENSAO"]);
            $attachment = new Attachment($report, $fileName);
            $attachment->setIsStringAttachment(true);
                
            $attachments[] = $attachment;
            
            $attOnBody.= "<a href= '".str_replace("PDF", "pdf", $report)."'>" . $dadosEmail["DSLINK"] . "</a><br/><br/>";
        }
        
        $message = $attOnBody.$message;
        
        $footer = "<div style=\"font-size: 10px; margin-top: 20px; word-wrap: break-word;\">" . $dadosEmail["MSGOPL"] . "<br/>" . $dadosEmail["MSGDATESENT"] . DateUtil::getDataAtual() . "</div>";
        
        $bodyMessage = "<table>
                            <tr>
                                <td class='description'>
                                    <br/>" . $message .
                                "</td>
                            </tr>
                            <tr class='footer'>
                                <td>
                                    <br> <p>" . $footer . "</p>
                                </td>
                            </tr>
                        </table>";
        
        return [$bodyMessage, $attachments, $clearAttachments];
    }
    
    public function retornaCookieEmail(DTO\Request\Row $request, DTO\Response $response) {
        $row[] = "NOME_ARQUIVO_OPL";
        $row[] = "BODY_MAIL_ANEXO_OPL";
        $row[] = "SUBJECT_MAIL_OPL";
        $row[] = "SEND_TO_OPERATOR_OPL";
        $row[] = "EMAILS_LIST_OPL";
        
        foreach($row as $msg) {
            $cookie[] = $this->oplUtilities->retornaCookieEmail($msg);
        }
        
        $response->addMessage(new Message($cookie));
    }
}